	Options	/Extend

C+
	Integer Function Check_New_Password (password, user, enc_pass, salt,
     &					     algorithm, min_length, flags)

C   Check the new password.  The requirements are:
C		1. Only letters, numbers, $, _
C		2. A length of at least 'min_length'
C		3. It must be different from the old password.
C		4. Not in the dictionary (unless the user is exempted).
C		5. Currently we can't check the user's history.

	Implicit	None
	Character *(*)	password, user
	Character *32	entry_name
	Integer		enc_pass (2), test_enc (2), salt, algorithm
	Integer		min_length, flags, dict, iostatus, status
	Logical		first_time /.true./, ok
	Include		'($UAIDEF)'
	Include		'($RMSDEF)'
	Integer		Sys$Hash_Password
	Save		first_time, dict
	External	Cngpass__Cantchange, Cngpass__Genpwd
	External	Cngpass__Tooshort, Cngpass__Internal
	External	Cngpass__indict, Cngpass__Same

C  Is the user allowed to change his password ?

	If (Ibits (flags, UAI$V_Lockpwd, 1)) then
		Check_New_Password = %Loc (Cngpass__Cantchange)
		Return
	End If

C  Is he restricted to generated passwords ?   

	If (Ibits (flags, UAI$V_Genpwd, 1)) then         
		Check_New_Password = %Loc (Cngpass__Genpwd)
		Return
	End If

C  Is the new password long enough ?

	If (Len (password) .lt. min_length) then
		Check_New_Password = %Loc (Cngpass__Tooshort)
		Return
	End If

C  See if the new password is the same as the old one.

C  We must hash the new password to compare.

	ok = Sys$Hash_Password (password, algorithm, salt, user, test_enc)

	If (.not. ok) then		! oops
		Check_New_Password = %Loc (Cngpass__Internal)
		Return
	End If

	If (test_enc (1) .eq. enc_pass (1) .and.
     &	    test_enc (2) .eq. enc_pass (2)) then
		Check_New_Password = %Loc (Cngpass__Same)
		Return
	End If

C   Is it in the dictionary ?

	If (.not. Ibits (flags, UAI$V_Dispwddic, 1)) then

C   Open the dictionary file

		If (first_time) then
			Call Lib$Get_Lun (dict)
			first_time = .false.
		End If

		Open (dict, name='SYS$SHARE:VMS$PASSWORD_DICTIONARY.DATA',
     &			readonly, shared, iostat=iostatus, status='old',
     &			organization='indexed',access='keyed',
     &			form='unformatted')
		If (iostatus .ne. 0) go to 80		! I/O error

		Read (dict, key=password, iostat=iostatus) entry_name

		If (iostatus .eq. 0) then		! record was found
			Close (dict)
			Check_New_Password = %Loc (Cngpass__indict)
			Return
		Else				! record not found
			Close (dict)
			Call Errsns (,,,, status)
			If (status .ne. RMS$_Eof) go to 80
		End If

	End If

C   Can't check history for the moment.

	Check_New_Password = .true.
	Return

C  I/O error occurred

80	Call ERRSNS (,,,, status)
	Check_New_Password = status
	End
