C+
	Program		Change_Password
C
C   Program to set a user password, accepting old and new passwords on the
C   command line.  Since this is to be used via a WEB CGI it also has the
C   option of setting DCL symbols instead of returning error codes.
C
C   The program verifies the password, and does intrusion detection.  It
C   can be installed with SYSPRV if it is to be used by non-system users (it
C   will only change a password if the old one matches).
C
C   It also writes a log of all change requests.
C-

	Implicit	None
	Character *(*)	command
	Parameter	(command = 'SETPASS')
	Character *32	this_user, target_user, old_password, new_password
	Character *255	text, error
	Character *1024	tty, port_name, remote_ip_name
	Integer		job_type, l_tty, l_portname, l_node
	Integer		l_caller, l_target, l_newpass, l_oldpass, salt
	Integer		algorithm, min_length, node, address, flags, uic
	Integer		length, l_error, Cli$DCL_Parse, l_remip
	Integer *4	enc_pass (2), new_pass (2), privmask (2)
	Logical		symbol, password_correct, ok, allowed
	Logical		Cli$Present, Get_User_Info, web_flag
	Logical		Check_Old_Password, Check_New_Password
	Integer		Sys$Scan_Intrusion, Set_Password, Check_Identifier
	Include		'($JPIDEF)'
	Include		'($SSDEF)'
	External	CIA$M_Real_Username, Secsrv$_Intruder, Setpass_CLD
	External	Cngpass__Authfail, Cngpass__Internal

C   Get and parse the command line.

	Call Lib$Get_Foreign (text, 'Command:', length)
	text = command // ' ' // text (1:length)
	length = Len (command) + 1 + length

	ok = Cli$DCL_Parse (text (1:length), SETPASS_CLD)
	If (.not. ok) Call Sys$Exit (%Val (ok))

C   Find what type of output we need, and whether we are a WEB daemon

	symbol = Cli$Present ('SYMBOL')
	web_flag = Cli$Present ('WEB')

C   Get the current user

	Call Get_User (this_user, l_caller, uic, privmask, tty, l_tty,
     &			 job_type, port_name, l_portname)

C   Check he has the right privileges for a /WEB qualifier

	If (web_flag) then
	   ok = Check_Identifier ('WEB_DAEMON', uic)
	   If (.not. ok) Call Sys$Exit (%Val (ok))

	   Call Lib$Get_Symbol ('WWW_REMOTE_HOST', remote_ip_name, l_remip)

	   If (l_remip .eq. 0) then
		Call Lib$Get_Symbol ('WWW_REMOTE_ADDR', remote_ip_name, l_remip)
	   End If
	End If

C   Store privilege mask for subsequent enable/disable calls

	Call Set_Privilege_Mask (privmask)

C   Turn off privileges  (we turn them on only when we need them)

	Call Disable_Privileges

C   Get the target user (defaults to himself)

	If (Cli$Present ('USERNAME')) then
		Call Cli$Get_Value ('USERNAME', target_user, l_target)
	Else
		target_user = this_user
		l_target = l_caller
	End If


C   Get the old and new passwords

	Call Cli$Get_Value ('OLD_PASSWORD', old_password, l_oldpass)
	Call Cli$Get_Value ('NEW_PASSWORD', new_password, l_newpass)

C   Get info about the target user - his encrypted password, salt and
C   algorithm used, as well as his minimum password length and whether he
C   is allowed change the password.

	flags = 0		! assume user doesn't exist for the moment.

	ok = Get_User_Info (target_user (1:l_target), enc_pass, salt,
     &				algorithm, min_length, allowed)

	If (.not. ok) go to 802			! no such user

C   Check to see if the password is valid.  If it isn't then we will return
C   a non-specific authorization failure message.


	flags = %Loc (CIA$M_Real_Username)		! username does exist

	ok = Check_Old_Password (target_user (1:l_target), enc_pass, salt,
     &				 algorithm, old_password (1:l_oldpass))

	If (.not. ok) go to 802			! same vague error message

C   OK - the old password pans out.  First check to see if he is an
C   intruder.

	Call Enable_Privileges

	If (web_flag) then
	   ok = Sys$Scan_Intrusion (%Val (1), target_user (1:l_target),
     &				%Val (JPI$K_Network),,
     &				remote_ip_name (1:l_remip),
     &				target_user (1:l_target),,
     &				old_password (1:l_oldpass),,, %Val (flags))
	Else
	   ok = Sys$Scan_Intrusion (%Val (1), target_user (1:l_target),
     &				 %Val (job_type), tty (1:l_tty),
     &				port_name (1:l_portname),
     &				this_user (1:l_caller),,
     &				old_password (1:l_oldpass),,, %Val(flags))
	End If

	Call Disable_Privileges

C   If the scan routine returns an error, or if he is spotted as an intruder
C   then treat it as if the password was incorrect.

	If ((.not. ok) .or. ok .eq. %Loc (SECSRV$_Intruder)) go to 803

C   Check the syntax rules of the new password

D	Call Lib$Put_Output ('Checking new password ...')

	ok = Check_New_Password (new_password (1:l_newpass),
     &				 target_user (1:l_target), enc_pass, salt,
     &				 algorithm, min_length, allowed, new_pass,
     &				 text, length)
	If (.not. ok) go to 850

C   Everything looks OK - set the password.

D	Call Lib$Put_Output ('Setting new password')

	ok = Set_Password (target_user (1:l_target),
     &			     	new_password (1:l_newpass))
	If (.not. ok) go to 890			! Oops

C   Set this history record

	Call Set_History (target_user, new_pass, allowed)

C   Successful completion - return status and optionally DCL symbol

	If (symbol) call Lib$Set_Symbol ('SETPASS_RESULT', 'OK')
	Call Exit

C   Error with bad username. Just return an AUTHFAIL
C   code, as he could be trolling.
C
C801	If (symbol) then
C		Call Lib$Set_Symbol ('SETPASS_RESULT', 'FAIL')
C		Call Lib$Sys_Getmsg (%Loc (Cngpass__Authfail), l_error, error)
C		Call Lib$Set_Symbol ('SETPASS_REASON', error (1:l_error))
C		Call Exit
C	Else
C		Call Lib$Signal (Cngpass__Authfail)
C	End If

C   Bad password.  Not only do we give a failure message, but we must tell
C   the intrusion server about him.

802	Call Enable_Privileges

	If (web_flag) then
		ok = Sys$Scan_Intrusion (%Val (ok), target_user (1:l_target),
     &				%Val (JPI$K_Network),,
     &				remote_ip_name (1:l_remip),
     &				target_user (1:l_target),,
     &				old_password (1:l_oldpass),,, %Val (flags))
	Else
	   ok = Sys$Scan_Intrusion (%Val (ok), target_user (1:l_target),
     &				 %Val (job_type), tty (1:l_tty),
     &				 port_name (1:l_portname),
     &				 this_user (1:l_caller),,
     &				 old_password (1:l_oldpass),,, %Val(flags))
	End If

	Call Disable_Privileges

803	If (symbol) then
		Call Lib$Set_Symbol ('SETPASS_RESULT', 'FAIL')
		Call Lib$Sys_Getmsg (%Loc (Cngpass__Authfail), l_error, error)
		Call Lib$Set_Symbol ('SETPASS_REASON', error (1:l_error))
		Call Exit
	Else
		Call Lib$Signal (Cngpass__Authfail)
		Call Sys$Exit (%Val (SS$_Cancel))
	End If

C   Something wrong with the new password

850	If (symbol) then
		Call Lib$Sys_Getmsg (ok, l_error, error)
		Call Lib$Set_Symbol ('SETPASS_RESULT', 'FAIL')
		Call Lib$Set_Symbol ('SETPASS_REASON', error (1:l_error))
		Call Exit
	Else
		Call Lib$Signal (%Val (ok))
		Call Sys$Exit (%Val (SS$_Cancel))
	End If

C   Something went wrong with the $SETUAI command

890	If (symbol) then
		Call Lib$Set_Symbol ('SETPASS_RESULT', 'FAIL')
		Call Lib$Sys_Getmsg (%Loc (Cngpass__Internal), l_error, error)
		Call Lib$Set_Symbol ('SETPASS_REASON', error (1:l_error))
		Call Exit
	Else
		Call Lib$Signal (ok)
	      	Call Sys$Exit (%Val (SS$_Cancel))
	End If

	End
